import { HttpClient, HttpEventType, HttpHeaderResponse, HttpHeaders, HttpParams, HttpRequest } from '@angular/common/http';
import { throwError } from 'rxjs';
import { catchError, filter, map } from 'rxjs/operators';
import { Injectable, Directive, ElementRef, EventEmitter, HostListener, Input, Output, Renderer2, NgModule, defineInjectable, inject } from '@angular/core';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var NgxfUploaderService = /** @class */ (function () {
    function NgxfUploaderService(http) {
        this.http = http;
    }
    /**
     * @param {?} option
     * @return {?}
     */
    NgxfUploaderService.prototype.upload = /**
     * @param {?} option
     * @return {?}
     */
    function (option) {
        if ((option.files instanceof Blob) ||
            (option.files instanceof File) ||
            (option.files instanceof Array && option.files.length !== 0)) {
            return this.http.request(new HttpRequest(option.method || 'POST', option.url, getFormData(option), {
                headers: option.headers instanceof HttpHeaders ? option.headers : new HttpHeaders(option.headers),
                params: option.params instanceof HttpParams ? option.params : new HttpParams({ fromObject: option.params }),
                reportProgress: option.process,
                responseType: option.responseType,
                withCredentials: option.withCredentials
            })).pipe(filter(function (response) {
                if (option.process) {
                    return !(response instanceof HttpHeaderResponse || response.type === HttpEventType.DownloadProgress);
                }
                return (response.type === HttpEventType.Response);
            }), map(function (event) {
                switch (event.type) {
                    case HttpEventType.Sent:
                        return (/** @type {?} */ ({
                            status: 0 /* Uploading */,
                            percent: 0
                        }));
                    case HttpEventType.UploadProgress:
                        return (/** @type {?} */ ({
                            status: 0 /* Uploading */,
                            percent: Math.round(100 * event.loaded / event.total) || 0
                        }));
                    case HttpEventType.Response:
                        return (/** @type {?} */ ({
                            status: 1 /* Completed */,
                            percent: 100,
                            data: event.body
                        }));
                }
            }), catchError(function (error) {
                return throwError((/** @type {?} */ ({
                    status: 2 /* UploadError */,
                    data: error.error
                })));
            }));
        }
        return throwError((/** @type {?} */ ({ status: 3 /* FileNumError */ })));
    };
    NgxfUploaderService.decorators = [
        { type: Injectable, args: [{
                    providedIn: 'root'
                },] }
    ];
    /** @nocollapse */
    NgxfUploaderService.ctorParameters = function () { return [
        { type: HttpClient }
    ]; };
    /** @nocollapse */ NgxfUploaderService.ngInjectableDef = defineInjectable({ factory: function NgxfUploaderService_Factory() { return new NgxfUploaderService(inject(HttpClient)); }, token: NgxfUploaderService, providedIn: "root" });
    return NgxfUploaderService;
}());
/**
 * @param {?} option
 * @return {?}
 */
function getFormData(option) {
    /** @type {?} */
    var data = new FormData();
    if (option.fields) {
        Object.keys(option.fields).forEach(function (key) { return data.append(key, option.fields[key]); });
    }
    if ((option.files instanceof Blob) || (option.files instanceof File)) {
        data.append((/** @type {?} */ (option.filesKey)) || 'file', option.files, option.files['name'] || 'blob');
    }
    else {
        for (var i = 0; i < option.files.length; i++) {
            /** @type {?} */
            var key = 'file';
            // If it has key
            if (option.filesKey) {
                if (option.filesKey instanceof Array) {
                    key = option.filesKey[i % option.filesKey.length];
                }
                else {
                    key = option.filesKey;
                }
            }
            data.append(key, option.files[i], option.files[i]['name'] || "blob" + i);
        }
    }
    return data;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @param {?} files
 * @param {?} accept
 * @param {?} multiple
 * @param {?} option
 * @return {?}
 */
function emitOpload(files, accept, multiple, option) {
    if (multiple !== undefined) {
        return checkAllFile(files, accept, option);
    }
    if (files.length === 1) {
        return checkAllFile(files[0], accept, option);
    }
    return 0 /* NumError */;
}
/**
 * @param {?} file
 * @param {?} accept
 * @param {?} option
 * @return {?}
 */
function checkAllFile(file, accept, option) {
    if (file instanceof FileList) {
        /** @type {?} */
        var files_1 = [];
        /** @type {?} */
        var err_1;
        /** @type {?} */
        var allPass = Array.from(file).every(function (f) {
            err_1 = checkOneFile(f, accept, option);
            /** @type {?} */
            var noError = err_1 instanceof File;
            if (noError) {
                files_1.push(f);
            }
            // if skipInvalid, all pass
            return option.skipInvalid || noError;
        });
        if (!allPass) {
            return err_1;
        }
        return files_1;
    }
    return checkOneFile(file, accept, option);
}
/**
 * @param {?} file
 * @param {?} accept
 * @param {?} option
 * @return {?}
 */
function checkOneFile(file, accept, option) {
    if (!checkFileType(file, accept)) {
        return 1 /* TypeError */;
    }
    if (!checkFileSize(file, option)) {
        return 2 /* SizeError */;
    }
    return file;
}
/**
 * @param {?} file
 * @param {?} accept
 * @return {?}
 */
function checkFileType(file, accept) {
    if (accept) {
        /** @type {?} */
        var acceptedFilesArray = accept.split(',');
        return acceptedFilesArray.some(function (type) {
            /** @type {?} */
            var validType = type.trim();
            if (validType.charAt(0) === '.') {
                return file.name.toLowerCase().endsWith(validType.toLowerCase());
            }
            else if (/\/\*$/.test(validType)) {
                // This is something like a image/* mime type
                return file.type.replace(/\/.*$/, '') === validType.replace(/\/.*$/, '');
            }
            return file.type === validType;
        });
    }
    return true;
}
/**
 * @param {?} file
 * @param {?} option
 * @return {?}
 */
function checkFileSize(file, option) {
    if (option) {
        /** @type {?} */
        var size = file.size;
        /** @type {?} */
        var chkSize = option.size;
        if (chkSize &&
            ((chkSize.min && size < chkSize.min) ||
                (chkSize.max && size > chkSize.max))) {
            return false;
        }
    }
    return true;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var NgxfDropDirective = /** @class */ (function () {
    function NgxfDropDirective(_elm, _render) {
        this._elm = _elm;
        this._render = _render;
        this.uploadOutput = new EventEmitter();
        this.fileOption = {};
        this.dropClass = 'drop';
    }
    /**
     * @param {?} e
     * @return {?}
     */
    NgxfDropDirective.prototype.drop = /**
     * @param {?} e
     * @return {?}
     */
    function (e) {
        this.stopEvent(e);
        this._render.removeClass(this._elm.nativeElement, this.dropClass);
        this.uploadOutput.emit(emitOpload(e.dataTransfer.files, this.accept, this.multiple, this.fileOption));
    };
    /**
     * @param {?} e
     * @return {?}
     */
    NgxfDropDirective.prototype.dragenter = /**
     * @param {?} e
     * @return {?}
     */
    function (e) {
        this.stopEvent(e);
        this._render.addClass(this._elm.nativeElement, this.dropClass);
    };
    /**
     * @param {?} e
     * @return {?}
     */
    NgxfDropDirective.prototype.dragleave = /**
     * @param {?} e
     * @return {?}
     */
    function (e) {
        this.stopEvent(e);
        this._render.removeClass(this._elm.nativeElement, this.dropClass);
    };
    // prevent the file open event
    // prevent the file open event
    /**
     * @param {?} e
     * @return {?}
     */
    NgxfDropDirective.prototype.stopEvent = 
    // prevent the file open event
    /**
     * @param {?} e
     * @return {?}
     */
    function (e) {
        e.stopPropagation();
        e.preventDefault();
    };
    NgxfDropDirective.decorators = [
        { type: Directive, args: [{
                    selector: '[ngxf-drop]'
                },] }
    ];
    /** @nocollapse */
    NgxfDropDirective.ctorParameters = function () { return [
        { type: ElementRef },
        { type: Renderer2 }
    ]; };
    NgxfDropDirective.propDecorators = {
        uploadOutput: [{ type: Output, args: ['ngxf-drop',] }],
        fileOption: [{ type: Input, args: ['ngxf-validate',] }],
        dropClass: [{ type: Input, args: ['drop-class',] }],
        multiple: [{ type: Input }],
        accept: [{ type: Input }],
        drop: [{ type: HostListener, args: ['drop', ['$event'],] }],
        dragenter: [{ type: HostListener, args: ['dragover', ['$event'],] }, { type: HostListener, args: ['dragenter', ['$event'],] }],
        dragleave: [{ type: HostListener, args: ['dragleave', ['$event'],] }]
    };
    return NgxfDropDirective;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var NgxfSelectDirective = /** @class */ (function () {
    function NgxfSelectDirective(_render, _elm) {
        this._render = _render;
        this._elm = _elm;
        this.uploadOutput = new EventEmitter();
        this.fileOption = {};
    }
    /**
     * @return {?}
     */
    NgxfSelectDirective.prototype.click = /**
     * @return {?}
     */
    function () {
        this.bindBeforeClick();
        this.fileElm.click();
    };
    /**
     * @return {?}
     */
    NgxfSelectDirective.prototype.ngAfterViewInit = /**
     * @return {?}
     */
    function () {
        this.fileElm = this._render.createElement('input');
        this._render.setAttribute(this.fileElm, 'type', 'file');
        this._render.setStyle(this.fileElm, 'display', 'none');
        this._render.appendChild(this._elm.nativeElement.parentNode, this.fileElm);
    };
    /**
     * @return {?}
     */
    NgxfSelectDirective.prototype.ngOnDestroy = /**
     * @return {?}
     */
    function () {
        // remove listen
        this.removeListen();
        this._render.removeChild(this._elm.nativeElement.parentNode, this.fileElm);
    };
    // beacuse bind before click, that can change input to change accept and validate
    // beacuse bind before click, that can change input to change accept and validate
    /**
     * @private
     * @return {?}
     */
    NgxfSelectDirective.prototype.bindBeforeClick = 
    // beacuse bind before click, that can change input to change accept and validate
    /**
     * @private
     * @return {?}
     */
    function () {
        var _this = this;
        if (this.multiple !== undefined) {
            this._render.setAttribute(this.fileElm, 'multiple', '');
        }
        else {
            this._render.removeAttribute(this.fileElm, 'multiple');
        }
        this._render.setAttribute(this.fileElm, 'accept', this.accept);
        this.removeListen();
        this.changeListen = this._render.listen(this.fileElm, 'change', function (e) {
            // when length is more than 0
            if (_this.fileElm.files.length) {
                _this.uploadOutput.emit(emitOpload(_this.fileElm.files, _this.accept, _this.multiple, _this.fileOption));
            }
            _this.fileElm.value = '';
            _this.removeListen();
        });
    };
    /**
     * @private
     * @return {?}
     */
    NgxfSelectDirective.prototype.removeListen = /**
     * @private
     * @return {?}
     */
    function () {
        if (this.changeListen) {
            this.changeListen();
        }
    };
    NgxfSelectDirective.decorators = [
        { type: Directive, args: [{
                    selector: '[ngxf-select]'
                },] }
    ];
    /** @nocollapse */
    NgxfSelectDirective.ctorParameters = function () { return [
        { type: Renderer2 },
        { type: ElementRef }
    ]; };
    NgxfSelectDirective.propDecorators = {
        uploadOutput: [{ type: Output, args: ['ngxf-select',] }],
        fileOption: [{ type: Input, args: ['ngxf-validate',] }],
        multiple: [{ type: Input }],
        accept: [{ type: Input }],
        click: [{ type: HostListener, args: ['click',] }]
    };
    return NgxfSelectDirective;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var NgxfUploaderModule = /** @class */ (function () {
    function NgxfUploaderModule() {
    }
    NgxfUploaderModule.decorators = [
        { type: NgModule, args: [{
                    declarations: [
                        NgxfSelectDirective,
                        NgxfDropDirective
                    ],
                    exports: [
                        NgxfSelectDirective,
                        NgxfDropDirective
                    ]
                },] }
    ];
    return NgxfUploaderModule;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { NgxfUploaderService, NgxfUploaderModule, NgxfDropDirective as ɵb, NgxfSelectDirective as ɵa };

//# sourceMappingURL=ngxf-uploader.js.map